# DESCRIPTION -- PHW Thumbnail creator Library

package Library_thumbnailmaker;

use warnings;
no warnings 'uninitialized';
$|++;
use strict;
use IO::File;
use Image::Magick;
use GD;
use POSIX;

our($VERSION);
$VERSION = '1.0';

sub bytes_to_k {
  my ($bytes) = @_;
  my ($size);
  $size = (int($bytes/1024));                

  if ($size < 1) {
    return 1;
  } else {
    return $size;
  }
}

sub image_identify {
  my ($filename, %params) = @_;

  my ($command, $result, $output, $height, $width, $format, $size, @output, $i, $image);

  if (-f "$filename") {
    $image = new Image::Magick;
    ($width, $height, $size, $format) = $image->Ping("$filename");
    undef ($image);

    $size = &Library_thumbnailmaker::bytes_to_k($size)."K";

    return ($format, $width, $height, $size);
  }
}

sub generate_thumbnail2 {
  my ($FORM, %params) = @_;

  my ($filename, $filepath, $suffix, $prefix, $maxheight, $maxwidth, $quality, $border, $resize, $jpg_data, $fh,
      $cache_dir, $base_dir, $rv, $dest, $source, $destwidth, $sourcewidth, $sourceheight, $destheight, $image, $borderfile);

  $filename = $params{'filename'};
  $filepath = $params{'filepath'};
  $maxwidth = $params{'maxwidth'};
  $maxheight = $params{'maxheight'};
  $quality = $params{'quality'};
  $prefix = $params{'prefix'};
  $suffix = $params{'suffix'};
  $border = $params{'border'};

  $cache_dir = "$filepath/.cache";

  if (-e $cache_dir && -d $cache_dir) {
    $image = new Image::Magick;
    $rv = $image->read("$filepath/$filename");
    warn "$rv" if "$rv";
    $rv = $image->Resize(geometry=>"$maxwidth"."x$maxheight", filter=>"lanczos");
    warn "$rv" if "$rv";
    $rv = $image->Set(quality=>$quality);
    warn "$rv" if "$rv";
    $rv = $image->Write("$cache_dir/$prefix$filename$suffix");
    warn "$rv" if "$rv";

    undef($image);
  }
}


sub generate_thumbnail {
  my ($FORM, %params) = @_;

  my ($filename, $filepath, $suffix, $prefix, $maxheight, $maxwidth, $quality, $border, $resize, $jpg_data, $fh,
      $cache_dir, $base_dir, $rv, $dest, $source, $destwidth, $sourcewidth, $sourceheight, 
      $destheight, $truecolor, $image, $borderfile, $format, @junk, $thumb2);

  $filename = $params{'filename'};
  $filepath = $params{'filepath'};
  $maxwidth = $params{'maxwidth'};
  $maxheight = $params{'maxheight'};
  $quality = $params{'quality'};
  $prefix = $params{'prefix'};
  $suffix = $params{'suffix'};
  $border = lc($params{'border'});
  $truecolor = 1;

  $cache_dir = "$filepath/.cache";

  #print "DEBUG: $cache_dir<BR>\n";

  $thumb2 = 0;
  unless (-e $cache_dir) {
    &Library_thumbnailmaker::create_subdir($cache_dir,0755);
  }

  if ($maxwidth eq "") {
    $maxwidth = "80";
  }

  if ($maxheight eq "") {
    $maxheight = "80";
  }


  if ($quality eq "") {
    $quality = "95";
  }

  if (-e $cache_dir && -d $cache_dir) {

    ($format, @junk) = &Library_thumbnailmaker::image_identify("$filepath/$filename");
    #print "DEBUG: $format<BR>\n";
    if (uc($format) eq "JPEG") {
      $image = GD::Image->newFromJpeg("$filepath/$filename", $truecolor);
    } elsif (uc($format) eq "PNG") {
      $image = GD::Image->newFromPng("$filepath/$filename", $truecolor); 
    } else {
      &Library_thumbnailmaker::generate_thumbnail2($FORM, filename=>$filename, 
                                                   filepath=>$filepath,
                                                   suffix=>$suffix,
                                                   prefix=>$prefix,
                                                   maxwidth=>$maxwidth,
                                                   maxheight=>$maxheight,
                                                   quality=>$quality,
                                                   border=>$border);
      $thumb2++;
    }
    if ($image and $thumb2 < 1) {
      ($sourcewidth, $sourceheight) = $image->getBounds();
 
      $destwidth = $maxwidth;
      $destheight = ceil(($sourceheight/$sourcewidth) * $destwidth);
      if ($destheight > $maxheight) {
        $destheight = $maxheight;
        $destwidth = ceil(($sourcewidth/$sourceheight) * $destheight);
      }
  
      $dest = "$cache_dir/$prefix$filename$suffix";
      $source = "$filepath/$filename";
  
      $resize = new GD::Image($destwidth,$destheight,$truecolor);
  
      $resize->copyResampled($image,0,0,0,0, $destwidth, $destheight, $sourcewidth, $sourceheight);

      ($destwidth, $destheight) = $resize->getBounds();
      #print "DEST: $dest<BR>\n";
      #print "SOURCE: $source<BR>\n";
      #print "DEBUG: Dest TrueColor ".$resize->isTrueColor()."<BR>\n";
      #print "DEBUG: Source TrueColor ".$image->isTrueColor()."<BR>\n";
      #print "DEBUG: Source Dims: $sourcewidth x $sourceheight<BR>\n";
      #print "DEBUG: Dest Dims: $destwidth x $destheight<BR>";
  
      $jpg_data = $resize->jpeg($quality);
  
      $fh = new IO::File(">$dest");
      binmode $fh;
      print $fh ($jpg_data);
      close($fh);
  
      undef($jpg_data);
      undef($resize);
      undef($image);

    }

    if ($border ne "") {
      #print "DEBUG: $border<BR>\n";
      $image = new Image::Magick;
      $rv = $image->Read("$cache_dir/$prefix$filename$suffix");
      warn "$rv" if "$rv";
      $borderfile = new Image::Magick;
      $rv = $borderfile->Read("/htdocs/local/gallery/borders/topleft-$border.gif");
      warn "$rv" if "$rv";
      $rv = $borderfile->Read("/htdocs/local/gallery/borders/topright-$border.gif");
      warn "$rv" if "$rv";
      $rv = $borderfile->Read("/htdocs/local/gallery/borders/bottomleft-$border.gif");
      warn "$rv" if "$rv";
      $rv = $borderfile->Read("/htdocs/local/gallery/borders/bottomright-$border.gif");
      warn "$rv" if "$rv";
      $rv = $image->Composite(compose=>"Over",gravity=>"NorthWest",image=>$borderfile->[0]);
      warn "$rv" if "$rv";
      $rv = $image->Composite(compose=>"Over",gravity=>"NorthEast",image=>$borderfile->[1]);
      warn "$rv" if "$rv";
      $rv = $image->Composite(compose=>"Over",gravity=>"SouthWest",image=>$borderfile->[2]);
      warn "$rv" if "$rv";
      $rv = $image->Composite(compose=>"Over",gravity=>"SouthEast",image=>$borderfile->[3]);
      warn "$rv" if "$rv";
      $rv = $image->Set(quality=>$quality);
      warn "$rv" if "$rv";

      undef($borderfile);

      $rv = $image->Write("jpg:$cache_dir/$prefix$filename$suffix");
      warn "$rv" if "$rv";
      #print "DEBUG: $rv<BR>\n";

      undef($image);

    }
  }
}

sub create_subdir {
  my ($path, $mode) = @_;

  if ("$mode" eq "") {
    $mode = 0755;
  }

  unless (mkdir $path, $mode) {
    print "<FONT COLOR=RED>Directory Creation Failed! $!.</FONT><BR>";
  }
}

1;

__END__
